//  LensSpaces.c
//
//  Writes a .gen file for a lens space L(p,q),
//  for use with the CurvedSpaces software.

#include <stdio.h>
#include <math.h>

#define PI  3.14159265358979323846
#define RH  (1.414213562373095 / 2)

typedef double  SO4Matrix[4][4];

//  The "obvious" generator for a lens space (the matrix m
//  in the code below) would put the Earth on the special axis.
//  To avoid this, we'll conjugate m to get mm = a*m*A, which
//  keeps the Earth off the special axis. 
static SO4Matrix    a = {
                            { RH,  0,  0, -RH },
                            {  0,  1,  0,   0 },
                            {  0,  0,  1,   0 },
                            { RH,  0,  0,  RH }
                        },
                    A = {
                            { RH,  0,  0,  RH },
                            {  0,  1,  0,   0 },
                            {  0,  0,  1,   0 },
                            {-RH,  0,  0,  RH }
                        };

static void MultiplySO4Matrices(SO4Matrix m, SO4Matrix n, SO4Matrix mn);


void main(void)
{
    FILE        *fp;
    int         p,
                q,
                i,
                j;
    char        theName[64];
    SO4Matrix   m,
                mm,
                temp;

    while (1)
    {
        printf("p ");
        scanf("%d", &p);
        if (p <= 0)
            break;
        printf("q ");
        scanf("%d", &q);

        sprintf(theName, "L(%d,%d).gen", p, q);
        fp = fopen(theName, "w");
        if (fp == NULL)
        {
            printf("Couldn't open output file %s.\n", theName);
            break;
        }

        m[0][0] = cos(q*2*PI/p);  m[0][1] = -sin(q*2*PI/p);  m[0][2] = 0.0;          m[0][3] =  0.0;
        m[1][0] = sin(q*2*PI/p);  m[1][1] =  cos(q*2*PI/p);  m[1][2] = 0.0;          m[1][3] =  0.0;
        m[2][0] = 0.0;            m[2][1] =  0.0;            m[2][2] = cos(2*PI/p);  m[2][3] = -sin(2*PI/p);
        m[3][0] = 0.0;            m[3][1] =  0.0;            m[3][2] = sin(2*PI/p);  m[3][3] =  cos(2*PI/p);

        MultiplySO4Matrices(a, m, temp);
        MultiplySO4Matrices(temp, A, mm);

        fprintf(fp, "3.15\n\n");
        for (i = 0; i < 4; i++)
        {
            for (j = 0; j < 4; j++)
                fprintf(fp, "%15.12f ", mm[i][j]);
            fprintf(fp, "\n");
        }

        fclose(fp);

        printf("Generator written to file %s.\n\n", theName);
    }
}


static void MultiplySO4Matrices(
    SO4Matrix   m,
    SO4Matrix   n,
    SO4Matrix   mn)
{
    int         i,
                j,
                k;
    SO4Matrix   temp;

    for (i = 0; i < 4; i++)
        for (j = 0; j < 4; j++)
        {
            temp[i][j] = 0.0;
            for (k = 0; k < 4; k++)
                temp[i][j] += m[i][k] * n[k][j];
        }

    for (i = 0; i < 4; i++)
        for (j = 0; j < 4; j++)
            mn[i][j] = temp[i][j];
}
