//  DirichletDomain-D3D.hpp
//
//       YOU DO NOT NEED TO READ THIS FILE TO UNDERSTAND
//             COMPUTER GRAPHICS IN CURVED SPACES.
//
//      It serves only to provide some interesting content
//           in the complete version of the program.

#pragma once

#include "Macros-D3D.h"

//  Opaque typedefs for winged edge polyhedron.
typedef struct WEVertex         WEVertex;
typedef struct WEEdge           WEEdge;
typedef struct WEFace           WEFace;
typedef struct WEPolyhedron     WEPolyhedron;

//  A Direction is a vector whose direction is
//  well defined but whose length is arbitrary.
//  That is, two Directions are considered equivalent
//  iff they are positive multiples of one another.
//  Note:  a Direction and its negative are *not* equivalent.
typedef double  Direction[4];

//  For most purposes the Dirichlet domain doesn't need
//  to know the SpaceType, but I wanted to sneak it in
//  to normalize the beam width consistently.
//  I know it's a kludge, but I'm keeping the original
//  typedef SpaceType in CurvedSpaces.cpp for the sake of
//  CurvedSpacesSimple.cpp, which won't use Dirichlet domains
//  and will keep a purely local definition of SpaceType.
typedef enum
{
    SpaceSphericalDD = 0,
    SpaceFlatDD,
    SpaceHyperbolicDD
} SpaceTypeDD;


class CDirichletDomain
{
public:

                    CDirichletDomain(   int         aNumMatrices,
                                        D3DXMATRIX  *aMatrixList,
                                        SpaceTypeDD aSpaceType);
    virtual         ~CDirichletDomain(void);

    virtual void    CenterCamera(D3DXMATRIX *aCameraPosition);

    virtual void    RenderBeams(    LPDIRECT3DDEVICE7       aDevice,
                                    D3DXMATRIX              *aPosition,
                                    D3DXMATRIX              *aCameraPosition,
                                    LPDIRECTDRAWSURFACE7    aTexture,
                                    DWORD                   aColor,
                                    int                     aFogRange,
                                    DWORD                   (*aFogFunction)(float *, float *, int, float *));

protected:

    WEPolyhedron    *itsPolyhedron;

    void    FreePolyhedron(void);
    void    MakeCube(void);
    void    MakeEmptyPolyhedron(void);
    void    SetInitialCube(void);
    void    SlicePolyhedron(int aNumMatrices, D3DXMATRIX *aMatrixList);
    void    IntersectWithHalfspace(D3DXMATRIX *aMatrix, double *aWorstError);
    void    SplitEdge(WEEdge *anOldEdge, Direction aCutPoint);
    void    CutFaceIfNecessary(WEFace *aFace);
    void    AllEdgesCounterclockwise(WEFace *aFace);
    void    RedirectEdge(WEEdge *anEdge);
    void    InstallNewFace(D3DXMATRIX *aMatrix, Direction aNormal);
    BOOL    TemporaryFacesRemain(void);
    void    AssignMates(void);
    BOOL    MatrixIsApproximatelyIdentity(D3DXMATRIX *aMatrix);
    void    CountCells(void);
    void    VerifyFaces(void);
    void    AssignHues(void);
    void    PrepareBeams(SpaceTypeDD aSpaceType);
    void    InterpolateEdge(D3DXVECTOR4 aNearPoint, D3DXVECTOR4 aFarPoint, SpaceTypeDD aSpaceType, D3DLVERTEX *aBeamVertex);
    void    Normalize(D3DXVECTOR4 *aVector, SpaceTypeDD aSpaceType);
    double  InnerProduct(D3DXVECTOR4 u, D3DXVECTOR4 v, SpaceTypeDD aSpaceType);
    double  ArcCosh(double x);
};
