//  SpinningEarth-D3D.hpp

#pragma once

#include "Macros-D3D.h"

//  REFINEMENT_LEVEL tells how finely the triangulation will
//  be subdivided.
//
//      If REFINEMENT_LEVEL is 0 the Earth will be
//          triangulated as a plain octahedron (8 faces).
//
//      If REFINEMENT_LEVEL is 1 the Earth will be
//          triangulated as the first subdivision (32 faces).
//
//      If REFINEMENT_LEVEL is 2 the Earth will be
//          triangulated as the second subdivision (128 faces).
//
//      etc.
#define REFINEMENT_LEVEL    3


typedef struct
{
    int         itsNumVertices;
    D3DLVERTEX  *itsVertices;
    int         itsNumFaces;
    WORD        *itsFaces;
} Triangulation;


class CSpinningEarth
{
public:

                    CSpinningEarth(void);
    virtual         ~CSpinningEarth(void);

    virtual void    Update(double anElapsedTime);
    virtual void    Render( LPDIRECT3DDEVICE7       aDevice,
                            D3DXMATRIX              *aPosition,
                            D3DXMATRIX              *aCameraPosition,
                            LPDIRECTDRAWSURFACE7    aNorthernTexture,
                            LPDIRECTDRAWSURFACE7    aSouthernTexture,
                            DWORD                   aColor,
                            int                     aFogRange,
                            DWORD                   (*aFogFunction)(float *, float *, int, float *));

protected:
    double          itsRotationAngle;
    Triangulation   itsSubdivisions[REFINEMENT_LEVEL + 1];

    void            InitOctahedron(Triangulation *aTriangulation);
    void            SubdivideTriangulation(Triangulation *aTriangulation, Triangulation *aSubdivision);
    void            ComputeTextureCoordinates(float v[3], D3DLVERTEX *aVertex);
};
