//  SpinningEarth-GL.hpp

#pragma once

#include "Macros-GL.h"

//  REFINEMENT_LEVEL tells how finely the triangulation will
//  be subdivided.
//
//      If REFINEMENT_LEVEL is 0 the Earth will be
//          triangulated as a plain octahedron (8 faces).
//
//      If REFINEMENT_LEVEL is 1 the Earth will be
//          triangulated as the first subdivision (32 faces).
//
//      If REFINEMENT_LEVEL is 2 the Earth will be
//          triangulated as the second subdivision (128 faces).
//
//      etc.
#define REFINEMENT_LEVEL    3

typedef GLdouble    Vertex[3],
                    TextureUV[2];

typedef struct
{
    int         itsNumVertices;
    Vertex      *itsVertices;
    TextureUV   *itsTextureCoordinates;
    int         itsNumFaces;
    GLuint      *itsFaces;
    GLuint      itsDisplayList;
} Triangulation;


class CSpinningEarth
{
public:

                    CSpinningEarth(void);
    virtual         ~CSpinningEarth(void);

    virtual void    Update(double anElapsedTime);

    virtual void    MakeDisplayLists(GLuint aNorthernTexture, GLuint aSouthernTexture);
    virtual void    FreeDisplayLists(void);
    virtual void    RenderDisplayList(float aFogFactor, float aDistance);

protected:
    double          itsRotationAngle;
    Triangulation   itsSubdivisions[REFINEMENT_LEVEL + 1];

    void            InitOctahedron(Triangulation *aTriangulation);
    void            SubdivideTriangulation(Triangulation *aTriangulation, Triangulation *aSubdivision);
    void            ComputeTextureCoordinates(Vertex v, TextureUV t);
};
