//  SceneryObjectCube-D3D.cpp

#include "SceneryObjectCube-D3D.hpp"

#define RADIANS_PER_SECOND  (D3DX_PI/4)
#define CUBE_SIZE           0.05f



void CSceneryObjectCube::Update(double anElapsedTime)
{
    itsRotationAngle += RADIANS_PER_SECOND * anElapsedTime;

    if (itsRotationAngle > 2*D3DX_PI)
        itsRotationAngle -= 2*D3DX_PI;
}


void CSceneryObjectCube::Render(LPDIRECT3DDEVICE7 aDevice)
{
    D3DXMATRIX  theRotation,
                theWorldMatrix;
    D3DLVERTEX  theVertices[4];
    int         i,
                j,
                k;

    static DWORD    theColor[3][2] = 
    {
        {0xFFFF0000, 0xFF00FFFF},
        {0xFF00FF00, 0xFFFF00FF},
        {0xFF0000FF, 0xFFFFFF00}
    };
    static float    theFacePair[2][4][3] =
    {
        {
            {-CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE},
            {-CUBE_SIZE, +CUBE_SIZE, -CUBE_SIZE},
            {+CUBE_SIZE, +CUBE_SIZE, -CUBE_SIZE},
            {+CUBE_SIZE, -CUBE_SIZE, -CUBE_SIZE}
        },
        {
            {-CUBE_SIZE, -CUBE_SIZE, +CUBE_SIZE},
            {+CUBE_SIZE, -CUBE_SIZE, +CUBE_SIZE},
            {+CUBE_SIZE, +CUBE_SIZE, +CUBE_SIZE},
            {-CUBE_SIZE, +CUBE_SIZE, +CUBE_SIZE}
        }
    };

    try
    {
        //  Set the world matrix.
        D3DXMatrixRotationY(&theRotation, (float) itsRotationAngle);
        D3DXMatrixMultiply(&theWorldMatrix, &theRotation, &itsPosition);
        D3DTRY(aDevice->SetTransform(D3DTRANSFORMSTATE_WORLD, theWorldMatrix))

        //  Draw the cube.
        for (i = 0; i < 3; i++)
            for (j = 0; j < 2; j++)
            {
                for (k = 0; k < 4; k++)
                {
                    theVertices[k].x        = theFacePair[j][k][(0+i)%3];
                    theVertices[k].y        = theFacePair[j][k][(1+i)%3];
                    theVertices[k].z        = theFacePair[j][k][(2+i)%3];
                    theVertices[k].color    = theColor[i][j];
                    theVertices[k].specular = 0xFFFFFFFF;   //  unused
                    theVertices[k].tu       = 0.0f;         //  unused
                    theVertices[k].tv       = 0.0f;         //  unused
                }

                D3DTRY(aDevice->DrawPrimitive(
                            D3DPT_TRIANGLEFAN,
                            D3DFVF_LVERTEX,
                            theVertices,
                            4,
                            0))
            }
    }
    catch (CException *anException)
    {
        MessageBox(NULL, anException->Message(), "Rendering Error in SceneryObjectCube", MB_OK);
        SAFE_DELETE(anException)
    }
    catch (...)
    {
        MessageBox(NULL, "Unknown exception.", "Rendering Error in SceneryObjectCube", MB_OK);
    }
}
