//  SceneryObjectOctahedron-D3D.cpp

#include "SceneryObjectOctahedron-D3D.hpp"

#define RADIANS_PER_SECOND  (D3DX_PI/4)
#define OCTAHEDRON_SIZE     0.07f

void CSceneryObjectOctahedron::Update(double anElapsedTime)
{
    itsRotationAngle += RADIANS_PER_SECOND * anElapsedTime;

    if (itsRotationAngle > 2*D3DX_PI)
        itsRotationAngle -= 2*D3DX_PI;
}


void CSceneryObjectOctahedron::Render(LPDIRECT3DDEVICE7 aDevice)
{
    D3DXMATRIX  theRotation,
                theWorldMatrix;

    static D3DLVERTEX   theVertices[6] =
    {
        D3DLVERTEX(D3DVECTOR(-OCTAHEDRON_SIZE,         0,                0       ), 0xFFFF0000, 0xFFFFFFFF, 0.0f, 0.0f),
        D3DLVERTEX(D3DVECTOR(+OCTAHEDRON_SIZE,         0,                0       ), 0xFFFF0000, 0xFFFFFFFF, 0.0f, 0.0f),
        D3DLVERTEX(D3DVECTOR(        0,        -OCTAHEDRON_SIZE,         0       ), 0xFF00FF00, 0xFFFFFFFF, 0.0f, 0.0f),
        D3DLVERTEX(D3DVECTOR(        0,        +OCTAHEDRON_SIZE,         0       ), 0xFF00FF00, 0xFFFFFFFF, 0.0f, 0.0f),
        D3DLVERTEX(D3DVECTOR(        0,                0,        -OCTAHEDRON_SIZE), 0xFF0000FF, 0xFFFFFFFF, 0.0f, 0.0f),
        D3DLVERTEX(D3DVECTOR(        0,                0,        +OCTAHEDRON_SIZE), 0xFF0000FF, 0xFFFFFFFF, 0.0f, 0.0f),
    };
    static WORD theFaces[3*8] =
    {
        0, 3, 4,
        3, 1, 4,
        1, 2, 4,
        2, 0, 4,
        3, 0, 5,
        1, 3, 5,
        2, 1, 5,
        0, 2, 5
    };

    try
    {
        //  Set the world matrix.
        D3DXMatrixRotationX(&theRotation, (float) itsRotationAngle);
        D3DXMatrixMultiply(&theWorldMatrix, &theRotation, &itsPosition);
        D3DTRY(aDevice->SetTransform(D3DTRANSFORMSTATE_WORLD, theWorldMatrix))

        //  Draw the octahedron.
        D3DTRY(aDevice->DrawIndexedPrimitive(
                    D3DPT_TRIANGLELIST,
                    D3DFVF_LVERTEX,
                    theVertices,
                    6,
                    theFaces,
                    3*8,
                    0))
    }
    catch (CException *anException)
    {
        MessageBox(NULL, anException->Message(), "Rendering Error in SceneryObjectOctahedron", MB_OK);
        SAFE_DELETE(anException)
    }
    catch (...)
    {
        MessageBox(NULL, "Unknown exception.", "Rendering Error in SceneryObjectOctahedron", MB_OK);
    }
}
