//  SceneryObjectOctahedron-GL.cpp

#include "SceneryObjectOctahedron-GL.hpp"

#define DEGREES_PER_SECOND  45
#define OCTAHEDRON_SIZE     0.07f


CSceneryObjectOctahedron::CSceneryObjectOctahedron(void)
{
    itsRotationAngle = 0.0;
}


void CSceneryObjectOctahedron::Update(double anElapsedTime)
{
    itsRotationAngle += DEGREES_PER_SECOND * anElapsedTime;

    if (itsRotationAngle > 360.0)
        itsRotationAngle -= 360.0;
}


void CSceneryObjectOctahedron::Render(void)
{
    static GLfloat theVertices[6*3] =
    {
        -OCTAHEDRON_SIZE,         0,                0,
        +OCTAHEDRON_SIZE,         0,                0,
                0,        -OCTAHEDRON_SIZE,         0,
                0,        +OCTAHEDRON_SIZE,         0,
                0,                0,        -OCTAHEDRON_SIZE,
                0,                0,        +OCTAHEDRON_SIZE,
    };
    static GLfloat theColors[6*3] =
    {
        1, 0, 0,
        1, 0, 0,
        0, 1, 0,
        0, 1, 0,
        0, 0, 1,
        0, 0, 1
    };
    static GLuint theFaces[3*8] =
    {
        0, 3, 4,
        3, 1, 4,
        1, 2, 4,
        2, 0, 4,
        3, 0, 5,
        1, 3, 5,
        2, 1, 5,
        0, 2, 5
    };

    //  Get ready.
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();

    //  Rotate the octahedron about the y-axis, then move it to its position.
    //  (Note:  OpenGL accepts these instructions in reverse order.)
    glMultMatrixf((float *) &itsPosition.m);
    glRotated(itsRotationAngle, 0, 1, 0);

    //  Draw the octahedron.
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_COLOR_ARRAY);
    glVertexPointer(3, GL_FLOAT, 0, theVertices);
    glColorPointer (3, GL_FLOAT, 0, theColors  );
//  If you have OpenGL 1.2, glDrawRangeElements() may be better than glDrawElements().
//  glDrawRangeElements(GL_TRIANGLES, 0, 5, 8*3, GL_UNSIGNED_INT, theFaces);
    glDrawElements(GL_TRIANGLES, 8*3, GL_UNSIGNED_INT, theFaces);
    glDisableClientState(GL_COLOR_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);

    //  Clean up.
    glPopMatrix();
}
